import { AxiosInstance } from 'axios';
export declare class ShopifyCollectionService {
    private axiosInstance;
    private ShopifyProductService;
    constructor(axiosInstance: AxiosInstance);
    create: (collection: ICollection) => Promise<ICollection>;
    /**
     * Creates a collect at the end of the collection
     *
     * @param  {ICollect} collect
     * @returns Promise
     */
    createCollect: (collect: ICollect) => Promise<ICollect>;
    /**
     * Creates a collect at a certain position in the collection
     *
     * @param  {collectionId}
     * @param  {productId}
     * @param  {position} Position within the collection
     * @returns Promise
     */
    createCollectAtPosition: (collectionId: number, productId: number, position: number) => Promise<void>;
    /**
     * Removes all collects from Shopify.
     *
     * @param  {number} collectionId
     * @param  {number} interval in ms of every delete call to Shopify. Set to 0 or leave it empty to not queue the calls.
     * @returns Promise
     */
    removeCollects: (collectionId: number, interval?: number) => Promise<void>;
    /**
     * Removes a collect from a Shopify collection.
     *
     * @param  {number} collectId
     * @returns Promise
     */
    removeCollect: (collectId: number) => Promise<void>;
    getCollects: (collectionId: number) => Promise<ICollect[]>;
    get: (num: number) => Promise<IProduct[]>;
    getAll: () => Promise<ICollection[]>;
    delete: (id: number) => Promise<void>;
    deleteProductsWithTag: (collectionId: number, productTag: string) => Promise<void>;
    update: (collection: ICollection) => Promise<void>;
    publish: (id: number) => Promise<void>;
    unpublish: (id: number) => Promise<void>;
}
