"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const ErrorHelper_1 = require("./ErrorHelper");
class ShopifyCustomerService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.getById = (customerId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Get customer By Id ${customerId}`);
                    const response = yield this.axiosInstance.get(`/customers/${customerId}.json`);
                    Logger_1.Logger.info('Customer found', response.data.customer);
                    resolve(response.data.customer);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getByEmail = (email) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Get customers By Email ${email}`);
                    const response = yield this.axiosInstance.get(`/customers/search.json?query=email:${email}`);
                    Logger_1.Logger.info('Customers found', response.data.customers);
                    resolve(response.data.customers);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         *
         * Searches using /customers/search.json?query= and returns the customers that has the tag doing a filter on the endpoint results.
         *
         * @param  {string} tag to search for.
         * @returns Promise
         */
        this.getByTag = (tag) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Get customers By Tag ${tag}`);
                    const response = yield this.axiosInstance.get(`/customers/search.json?query=${tag.trim()}`);
                    const customers = response.data.customers;
                    Logger_1.Logger.info('Initial Customers found', response.data.customers);
                    const foundCustomers = [];
                    for (let customer of customers) {
                        if (customer.tags && customer.tags.trim().toLowerCase().indexOf(tag.trim().toLowerCase()) > -1) {
                            foundCustomers.push(customer);
                        }
                    }
                    Logger_1.Logger.info('Customers found', response.data.customers);
                    resolve(foundCustomers);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         *
         * Searches using /customers/search.json?query= and returns the customers that contains the string in the last name field.
         *
         * @param  {string} text to search for.
         * @returns Promise
         */
        this.getByLastNameContains = (text) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Get customers By Last Name ${text}`);
                    const response = yield this.axiosInstance.get(`/customers/search.json?query=${text.trim()}`);
                    const customers = response.data.customers;
                    Logger_1.Logger.info('Initial Customers found', response.data.customers);
                    const foundCustomers = [];
                    for (let customer of customers) {
                        if (customer.last_name && customer.last_name.trim().toLowerCase().indexOf(text.trim().toLowerCase()) > -1) {
                            foundCustomers.push(customer);
                        }
                    }
                    Logger_1.Logger.info('Customers found', response.data.customers);
                    resolve(foundCustomers);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.create = (customer) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('Create customers', customer);
                    const response = yield this.axiosInstance.post('/customers.json', { customer: customer });
                    resolve(response.data.customer);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.update = (custormerId, customer) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Update customers ${custormerId}`, customer);
                    const response = yield this.axiosInstance.put(`/customers/${custormerId}.json`, JSON.stringify({ customer: customer }));
                    resolve(response.data.customer);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.get = (num) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getting customers`);
                    let customers = [];
                    const response = yield this.axiosInstance.get(`/customers/count.json`);
                    const count = response.data.count > num ? num : response.data.count;
                    while (customers.length < count) {
                        let customerGetURL = '/customers.json?limit=250&since_id=';
                        if (customers.length > 1) {
                            customerGetURL += customers[customers.length - 1].id;
                        }
                        else {
                            customerGetURL += '0';
                        }
                        const productsRetrieved = yield this.axiosInstance.get(customerGetURL);
                        customers = customers.concat(productsRetrieved.data.customers);
                    }
                    resolve(customers.slice(0, count));
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Adds an address to a customer.
         *
         * @param  {number} customerId - Customer Id
         * @param  {IAddress} address
         * @returns Promise
         */
        this.addAddress = (customerId, address) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Adding a address to customer ${customerId}`, address);
                    const response = yield this.axiosInstance.post(`/customers/${customerId}/addresses.json`, { address: address });
                    resolve(response.data.customer_address);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Adds an address to a customer as its default.
         *
         * @param  {number} customerId - Cusotmer Id
         * @param  {IAddress} address
         * @returns Promise
         */
        this.addAddressAsDefault = (customerId, address) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    let addressCreated = yield this.addAddress(customerId, address);
                    addressCreated = yield this.addressAsDefault(customerId, addressCreated.id);
                    resolve(addressCreated);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Replaces the default address of a customer for the address provided.
         *
         * @param  {number} customerId - Customer Id
         * @param  {IAddress} address
         * @returns Promise
         */
        this.replacesAddressAsDefault = (customerId, address) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const customer = yield this.getById(customerId);
                    const defaultAddressId = customer.default_address ? customer.default_address.id : null;
                    let addressCreated = yield this.addAddress(customerId, address);
                    addressCreated = yield this.addressAsDefault(customerId, addressCreated.id);
                    if (defaultAddressId) {
                        yield this.removeAddress(customerId, defaultAddressId);
                    }
                    else {
                        Logger_1.Logger.info(`There wasn't a default address`);
                    }
                    resolve(addressCreated);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Converts an address of a customer to its default.
         *
         * @param  {number} customerId - Customer Id
         * @param  {IAddress} addressId
         * @returns Promise
         */
        this.addressAsDefault = (customerId, addressId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Address ${addressId} of customer ${customerId} as default`);
                    const response = yield this.axiosInstance.put(`/customers/${customerId}/addresses/${addressId}/default.json`, {});
                    resolve(response.data.customer_address);
                }
                catch (error) {
                    Logger_1.Logger.error('Error setting address as default', error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Adds an address to a cusotmer.
         *
         * @param  {number} id - Cusotmer Id
         * @param  {IAddress} address with id field in
         * @returns Promise
         */
        this.updateAddress = (id, address) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Updating address ${address.id} to customer ${id}`, address);
                    const response = yield this.axiosInstance.put(`/customers/${id}/addresses/${address.id}.json`, { address: address });
                    resolve(response.data.customer_address);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Removes an address to a customer.
         *
         * @param  {number} customerId - Customer Id
         * @param  {number} addressId
         * @returns Promise
         */
        this.removeAddress = (customerId, addressId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Removing address ${addressId} for customer ${customerId}`);
                    yield this.axiosInstance.delete(`/customers/${customerId}/addresses/${addressId}.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Gets all metafields for a customer or empty array.
         *
         * @param  {number} id - Cusotmer Id
         * @returns Promise
         */
        this.getMetafields = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Getting metafields for customer id ->  ${id}`);
                    const response = yield this.axiosInstance.get(`/customers/${id}/metafields.json`);
                    resolve(response.data.metafields);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Gets one concrete metafield for a customer or null if it does not exists.
         *
         * @param  {number} id - Cusotmer Id
         * @param  {string} namespace
         * @param  {string} key
         * @returns Promise
         */
        this.getMetafield = (id, namespace, key) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Getting metafield for customer id ->  ${id} and namespace ${namespace} and key ${key}`);
                    const metafields = yield this.getMetafields(id);
                    let foundMetafield = null;
                    if (metafields && metafields.length > 0) {
                        foundMetafield = metafields.find(m => m.namespace === namespace && m.key === key);
                    }
                    resolve(foundMetafield);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Sends invite to a customer.
         *
         * @param  {number} customerId
         * @returns Promise
         */
        this.sendInvite = (customerId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Send invite to customer ->  ${customerId}`);
                    const response = yield this.axiosInstance.post(`/customers/${customerId}/send_invite.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Creates a metafield for a customer and return the created metafield.
         *
         * @param  {number} id
         * @param  {IMetafield} metafield
         * @returns Promise
         */
        this.postMetafield = (id, metafield) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Posting metafield for customer id ->  ${id}`);
                    const response = yield this.axiosInstance.post(`/customers/${id}/metafields.json`, { metafield: metafield });
                    resolve(response.data.metafield);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Updates a metafield for a customer and return the updated metafield. <br>
         * The functionality first retrieves the customer metafield to get the ID and then updates it.
         *
         * @param  {number} customerId
         * @param  {IMetafield} metafield.
         * @returns Promise
         */
        this.putMetafield = (customerId, metafield) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Updating a metafield for customer id ->  ${customerId}`);
                    const metafieldSaved = yield this.getMetafield(customerId, metafield.namespace, metafield.key);
                    if (metafieldSaved && metafieldSaved.id) {
                        Logger_1.Logger.info(`Existing Metafield. Updating a metafield for customer id ->  ${customerId}`);
                        const response = yield this.axiosInstance.put(`/customers/${customerId}/metafields/${metafieldSaved.id}.json`, { metafield: { id: metafieldSaved.id, value: metafield.value } });
                        resolve(response.data.metafield);
                    }
                    else {
                        Logger_1.Logger.info(`Metafield not found. Updating the metafield for customer id ->  ${customerId}`);
                        const createdMetafield = yield this.postMetafield(customerId, metafield);
                        resolve(createdMetafield);
                    }
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Deletes a metafield for a customer.
         *
         * @param  {number} id - Customer Id
         * @param  {number} metafieldId
         * @returns Promise
         */
        this.deleteMetafield = (id, metafieldId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Posting metafield for customer id ->  ${id}`);
                    yield this.axiosInstance.delete(`/customers/${id}/metafields/${metafieldId}.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
    }
}
exports.ShopifyCustomerService = ShopifyCustomerService;
//# sourceMappingURL=ShopifyCustomerService.js.map