"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const ErrorHelper_1 = require("./ErrorHelper");
class ShopifyDraftOrderService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.create = (order) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('Creating draft order', order);
                    const response = yield this.axiosInstance.post('/draft_orders.json', { draft_order: order });
                    resolve(response.data.draft_order);
                }
                catch (error) {
                    const extractedErrorFromResponse = ErrorHelper_1.default.getErrorFromResponse(error);
                    Logger_1.Logger.error('Error while creating draft order', error, extractedErrorFromResponse);
                    reject(Error(extractedErrorFromResponse));
                }
            }));
        });
        this.createAndSendInvoice = (order) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('Creating draft order and sending invice');
                    const orderCreated = yield this.create(order);
                    const response = yield this.axiosInstance.post(`/draft_orders/${orderCreated.id}/send_invoice.json`, { draft_order_invoice: {} });
                    resolve(orderCreated);
                }
                catch (error) {
                    const extractedErrorFromResponse = ErrorHelper_1.default.getErrorFromResponse(error);
                    Logger_1.Logger.error('Error while creating draft order and sending invoice', error, extractedErrorFromResponse);
                    reject(Error(extractedErrorFromResponse));
                }
            }));
        });
        /**
         * Updates Shipping Address in a draft order.
         *
         * @param  {number} orderId
         * @param  {IAddress} shippingAddress
         * @returns Promise
         */
        this.updateShippingAddress = (orderId, shippingAddress) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Updating draft order shipping address with id ${orderId}`, shippingAddress);
                    const response = yield this.axiosInstance.put(`/draft_orders/${orderId}.json`, { draft_order: { id: orderId, shipping_address: shippingAddress } });
                    resolve(response.data.draft_order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error updating order shipping address', error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
    }
}
exports.ShopifyDraftOrderService = ShopifyDraftOrderService;
//# sourceMappingURL=ShopifyDraftOrderService.js.map