import { AxiosInstance } from 'axios';
export declare class ShopifyInventoryService {
    private axiosInstance;
    constructor(axiosInstance: AxiosInstance);
    /**
     *
     * @param  {number} inventoryId
     * @param  {boolean} trackable. Inventory quantity is traqcked by shopify
     * @returns Promise
     */
    setItemToBe: (inventoryId: number, trackable: boolean) => Promise<IInventoryItem>;
    /**
     * Updates Inventory cost per item.
     *
     * @param  {number} inventoryId
     * @param  {number} cost. New cost per item
     * @returns Promise
     */
    setItemCostToBe: (inventoryId: number, cost: number) => Promise<IInventoryItem>;
    /**
     * Updates Inventory cost & sku per item.
     *
     * @param  {number} inventoryId
     * @param  {number} cost. New cost per item
     * @param  {string} sku. New sku per item
     * @returns Promise
     */
    setItemCostAndSkuToBe: (inventoryId: number, cost: number, sku: string) => Promise<IInventoryItem>;
    /**
     * Sets a quantity to the inventory level
     *
     * @param  {number} inventoryId
     * @param  {number} qty Absolute amount of Qty
     * @param  {number} locationId
     * @returns Promise
     */
    setQuantity: (inventoryId: number, qty: number, locationId: number) => Promise<IInventoryLevel>;
    /**
     * Adjusts quantity to the inventory level. So it adds or removes the quantity from the actual value
     *
     * @param  {number} inventoryId
     * @param  {number} qty Absolute amount of Qty
     * @param  {number} locationId
     * @returns Promise
     */
    adjustQuantity: (inventoryId: number, qtyToAdjust: number, locationId: number) => Promise<IInventoryLevel>;
    /**
     * Adjusts quantity to the inventory levels of variant. So it adds or removes the quantity from the actual value.
     * - If quantity is lower than the inventory level. It gets removed from it.
     * - If quantity is bigger from the inventory level but there are more than one level it tries to remove a bit of stock from each level.
     * - If quantity is bigger than the invenotry level and there isn't anymore levels then it removes the stock. A negative value will be placed
     *
     * @param  {number} inventoryId
     * @param  {number} qty Absolute amount of Qty
     * @param  {number} locationId
     * @returns Promise
     */
    adjustQuantityOfInventoryItem: (inventoryItemId: number, qtyToAdjust: number) => Promise<void>;
    getLevel: (inventoryId: number) => Promise<IInventoryLevel[]>;
    /**
     * Returns a list of all iventory Items of the product.
     *
     * @param  {IProduct} product
     * @returns Promise
     */
    getInventoryItemsOfProduct: (product: IProduct) => Promise<IInventoryItemWithVariant[]>;
}
