"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const ErrorHelper_1 = require("./ErrorHelper");
const ShopifyShopService_1 = require("./ShopifyShopService");
const CountryCodeService_1 = require("../services/CountryCodeService");
class ShopifyOrderService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.getFulfillments = (orderId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getFulfillments for order ID ${orderId}`);
                    const response = yield this.axiosInstance.get(`/orders/${orderId}/fulfillments.json`);
                    resolve(response.data.fulfillments);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getById = (orderId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getting order for ID ${orderId}`);
                    const response = yield this.axiosInstance.get(`/orders/${orderId}.json`);
                    resolve(response.data.order);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getAll = (orderstatus) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (!orderstatus) {
                        orderstatus = 'any';
                    }
                    Logger_1.Logger.info(`getting orders`);
                    let orders = [];
                    const response = yield this.axiosInstance.get(`/orders/count.json`);
                    const count = response.data.count;
                    Logger_1.Logger.info(`Getting ${count} orders`);
                    while (orders.length < count) {
                        let ordersGetURL = `/orders.json?status=${orderstatus}&limit=250&since_id=`;
                        if (orders.length > 1) {
                            ordersGetURL += orders[orders.length - 1].id;
                        }
                        else {
                            ordersGetURL += '0';
                        }
                        let currentPage = (parseInt((orders.length / 250).toFixed(0)) + 1);
                        Logger_1.Logger.info('Orders Get URL  -> ', ordersGetURL);
                        const ordersRetrieved = yield this.axiosInstance.get(ordersGetURL);
                        orders = orders.concat(ordersRetrieved.data.orders);
                        Logger_1.Logger.info('Orders retrieved -> ' + orders.length);
                        let nextPage = (parseInt((orders.length / 250).toFixed(0)) + 1);
                        if (currentPage === nextPage) {
                            Logger_1.Logger.info('Orders next page is the current page so, we are breaking the loop!');
                            break;
                        }
                    }
                    resolve(orders);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.createFulfillments = (orderId, orderLocationId, orderFulfillments, notifyCustomer, shipmentStatus) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Creating fulfillment for order ID ${orderId}`);
                    const fulfillment = { notify_customer: notifyCustomer, shipment_status: shipmentStatus, status: 'success', order_id: orderId, service: 'manual', location_id: orderLocationId };
                    fulfillment.line_items = [];
                    for (const fulfillmentLine of orderFulfillments) {
                        fulfillment.line_items.push({ id: fulfillmentLine.line_item_id, quantity: fulfillmentLine.quantity_delivered });
                    }
                    const response = yield this.axiosInstance.post(`/orders/${orderId}/fulfillments.json`, { fulfillment: fulfillment });
                    resolve(response.data.fulfillment);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while creating fulfillments for orderId ' + orderId, error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.completeFulfillment = (orderId, fullfilmentId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Compliting fulfillment for order ID ${orderId}`);
                    const response = yield this.axiosInstance.post(`/orders/${orderId}/fulfillments/${fullfilmentId}/complete.json`);
                    resolve(response.data.fulfillment);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while compliting fulfillment for orderId ' + orderId, error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.openFulfillment = (orderId, fullfilmentId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Opening fulfillment for order ID ${orderId}`);
                    const response = yield this.axiosInstance.post(`/orders/${orderId}/fulfillments/${fullfilmentId}/open.json`);
                    resolve(response.data.fulfillment);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while opening fulfillment for orderId ' + orderId, error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.cancelFulfillment = (orderId, fullfilmentId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Canceling fulfillment for order ID ${orderId}`);
                    const response = yield this.axiosInstance.post(`/orders/${orderId}/fulfillments/${fullfilmentId}/cancel.json`);
                    resolve(response.data.fulfillment);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while canceling fulfillment for orderId ' + orderId, error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.addAdditionalAttribute = (orderId, attributeName, attributeValue) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Adding attribute to order with ID ${orderId}`);
                    let additionalAttributes = [];
                    const orderFromShopify = yield this.getById(orderId);
                    if (orderFromShopify.note_attributes) {
                        additionalAttributes = orderFromShopify.note_attributes;
                        const attr = additionalAttributes.find(a => a.name === attributeName);
                        if (attr) {
                            attr.value = attributeValue;
                        }
                        else {
                            additionalAttributes.push({ name: attributeName, value: attributeValue });
                        }
                    }
                    else {
                        additionalAttributes.push({ name: attributeName, value: attributeValue });
                    }
                    Logger_1.Logger.info(`Adding attributes to order `, additionalAttributes);
                    const order = { id: orderId, note_attributes: additionalAttributes };
                    const response = yield this.axiosInstance.put(`/orders/${orderId}.json`, { order: order });
                    resolve(response.data.order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while adding note attribute to order with Id ' + orderId, error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.addAdditionalAttributes = (orderId, attributes) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Adding attributes to order with ID ${orderId}`);
                    let additionalAttributes = [];
                    const orderFromShopify = yield this.getById(orderId);
                    if (orderFromShopify.note_attributes) {
                        additionalAttributes = orderFromShopify.note_attributes;
                        for (const attribute of attributes) {
                            const attr = additionalAttributes.find(a => a.name === attribute.attributeName);
                            if (attr) {
                                attr.value = attribute.attributeValue;
                            }
                            else {
                                additionalAttributes.push({ name: attribute.attributeName, value: attribute.attributeValue });
                            }
                        }
                    }
                    else {
                        for (const attribute of attributes) {
                            additionalAttributes.push({ name: attribute.attributeName, value: attribute.attributeValue });
                        }
                    }
                    Logger_1.Logger.info(`Adding attributes to order `, additionalAttributes);
                    const order = { id: orderId, note_attributes: additionalAttributes };
                    const response = yield this.axiosInstance.put(`/orders/${orderId}.json`, { order: order });
                    resolve(response.data.order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while adding note attribute to order with Id ' + orderId, error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.setTags = (orderId, tags) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Adding tags to order with ID ${orderId}`);
                    const order = { id: orderId, tags: tags };
                    const response = yield this.axiosInstance.put(`/orders/${orderId}.json`, { order: order });
                    resolve(response.data.order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while adding tags to order with Id ' + orderId, error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.create = (order) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Creating order`, order);
                    const response = yield this.axiosInstance.post('/orders.json', { order: order });
                    resolve(response.data.order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while creating order', error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Updates Shipping Address in a order.
         *
         * @param  {number} orderId
         * @param  {IAddress} shippingAddress
         * @returns Promise
         */
        this.updateShippingAddress = (orderId, shippingAddress) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Updating order shipping address with id ${orderId}`, shippingAddress);
                    const response = yield this.axiosInstance.put(`/orders/${orderId}.json`, { order: { id: orderId, shipping_address: shippingAddress } });
                    resolve(response.data.order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error updating order shipping address', error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.cancel = (orderId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Cancelling order`, orderId);
                    const response = yield this.axiosInstance.post(`orders/${orderId}/cancel.json`, {});
                    resolve(response.data.order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while cancelling order', error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.applyWeightAndCountryBasedShippingToOrder = (order) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const shippingZones = yield this.shopifyShopService.getShippingRates();
                    let countryCode = order.shipping_address.country_code;
                    let country = order.shipping_address.country;
                    if (!countryCode && !country) {
                        countryCode = '*';
                    }
                    else if (!countryCode && country) {
                        countryCode = CountryCodeService_1.default.getCountryCode(country);
                    }
                    let totalWeight = order.line_items.reduce((a, b) => { if (b.requires_shipping) {
                        return (a + b.quantity * b.grams);
                    }
                    else {
                        return a;
                    } }, 0) / 1000;
                    const shippingZoneSelected = shippingZones.find(zone => (zone.countries.map(country => country.code).join(',') + ',').indexOf(countryCode + ',') > -1);
                    if (shippingZoneSelected) {
                        Logger_1.Logger.info(`Shipping Zone found for country ${countryCode}`, shippingZoneSelected);
                        if (shippingZoneSelected.weight_based_shipping_rates && shippingZoneSelected.weight_based_shipping_rates.length > 0) {
                            const shippingRateSelected = shippingZoneSelected.weight_based_shipping_rates.find(shippingRate => ((shippingRate.weight_low < totalWeight && shippingRate.weight_high > totalWeight) || shippingRate.weight_low === totalWeight));
                            if (shippingRateSelected) {
                                Logger_1.Logger.info('Adding SHIPPING RATE to order', shippingRateSelected);
                                order.shipping_lines = [{ code: shippingZoneSelected.name, title: shippingZoneSelected.name, price: shippingRateSelected.price, source: 'shopify', tax_lines: [] }];
                            }
                            else {
                                Logger_1.Logger.info(`No Shipping Rate found for weight ${totalWeight}`);
                            }
                        }
                        else {
                            Logger_1.Logger.info('Shipping Zone is not Weight based');
                        }
                    }
                    else {
                        Logger_1.Logger.info(`NO Shipping Zone found for country ${countryCode}`);
                    }
                    resolve(order);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while cancelling order', error);
                    reject(Error(error));
                }
            }));
        });
        this.getShippingByWeight = (country, weight) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    let shippingLine = null;
                    const shippingZones = yield this.shopifyShopService.getShippingRates();
                    const countryCode = CountryCodeService_1.default.getCountryCode(country);
                    const shippingZoneSelected = shippingZones.find(zone => (zone.countries.map(country => country.code).join(',') + ',').indexOf(countryCode + ',') > -1);
                    if (shippingZoneSelected) {
                        Logger_1.Logger.info(`Shipping Zone found for country ${countryCode}`, shippingZoneSelected);
                        if (shippingZoneSelected.weight_based_shipping_rates && shippingZoneSelected.weight_based_shipping_rates.length > 0) {
                            const shippingRateSelected = shippingZoneSelected.weight_based_shipping_rates.find(shippingRate => (shippingRate.weight_low <= weight && shippingRate.weight_high >= weight));
                            if (shippingRateSelected) {
                                Logger_1.Logger.info('Adding SHIPPING RATE to order', shippingRateSelected);
                                shippingLine = { code: shippingZoneSelected.name, title: shippingZoneSelected.name, price: shippingRateSelected.price, source: 'shopify', tax_lines: [] };
                            }
                            else {
                                Logger_1.Logger.info(`No Shipping Rate found for weight ${weight}`);
                            }
                        }
                        else {
                            Logger_1.Logger.info('Shipping Zone is not Weight based');
                        }
                    }
                    else {
                        Logger_1.Logger.info(`NO Shipping Zone found for country ${countryCode}`);
                    }
                    resolve(shippingLine);
                }
                catch (error) {
                    Logger_1.Logger.error('Error while cancelling order', error);
                    reject(Error(error));
                }
            }));
        });
        this.shopifyShopService = new ShopifyShopService_1.ShopifyShopService(axiosInstance);
    }
}
exports.ShopifyOrderService = ShopifyOrderService;
//# sourceMappingURL=ShopifyOrderService.js.map