"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const ErrorHelper_1 = require("./ErrorHelper");
class ShopifyPageService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.post = (title, bodyHtml, suffix) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Creating page with title ${title} with template ${suffix}`);
                    const response = yield this.axiosInstance.post('/pages.json', { page: { title: title, body_html: bodyHtml, template_suffix: suffix } });
                    resolve(response.data.page);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.put = (bodyHtml, pageId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Changing page ${pageId}`);
                    const changePageRequest = { body_html: bodyHtml };
                    const response = yield this.axiosInstance.put(`/pages/${pageId}.json`, { page: changePageRequest });
                    resolve(response.data.page);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Creates metafields for a page and return the created metafields.
         *
         * @param  {number} id
         * @param  {IMetafield} metafield
         * @returns Promise
         */
        this.postMetafields = (id, metafields) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const createdMetafields = [];
                    const erroredMetafields = [];
                    for (const metafield of metafields) {
                        try {
                            Logger_1.Logger.info(`Posting metafield for page id ->  ${id}`);
                            const response = yield this.axiosInstance.post(`/pages/${id}/metafields.json`, { metafield: metafield });
                            createdMetafields.push(response.data.metafield);
                        }
                        catch (e) {
                            erroredMetafields.push(metafield);
                            Logger_1.Logger.error('Failure while creating metafield for page resource', metafield);
                        }
                    }
                    if (createdMetafields.length != metafields.length) {
                        Logger_1.Logger.error(`Some metafields did fail`, erroredMetafields);
                        reject();
                    }
                    else {
                        resolve(createdMetafields);
                    }
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
    }
}
exports.ShopifyPageService = ShopifyPageService;
//# sourceMappingURL=ShopifyPageService.js.map