import { AxiosInstance } from 'axios';
export declare class ShopifyProductService {
    private axiosInstance;
    constructor(axiosInstance: AxiosInstance);
    create: (product: IProduct) => Promise<IProduct>;
    update: (product: IProduct, id: number) => Promise<IProduct>;
    getAll: () => Promise<IProduct[]>;
    getCount: () => Promise<number>;
    get: (num: number) => Promise<IProduct[]>;
    getInBatch: (sinceId: number) => Promise<IProductBatch>;
    /**
     * It will only return exact matches.
     */
    getByTitle: (title: string) => Promise<IProduct[]>;
    /**
     * It will only return exact matches.
     */
    getByHandle: (handle: string) => Promise<IProduct[]>;
    getByProductType: (type: string) => Promise<IProduct[]>;
    getById: (id: number) => Promise<IProduct>;
    delete: (id: number) => Promise<void>;
    /**
     * Gets All metafields of a product
     *
     * @param  {number} productId
     * @returns Promise
     */
    getMetafields: (productId: number) => Promise<IMetafield[]>;
    /**
    *
    * Adds or updates a metafield.
    *
    * @param  {number} id - product ID
    * @param  {IMetafield} metafield to be saved
    * @returns Promise
    */
    saveOrUpdateMetafield: (id: number, metafield: IMetafield) => Promise<IMetafield>;
    /**
     * Gets a metafield matching a product namespace and key.
     *
     * @param  {number} productId
     * @param  {string} namespace
     * @param  {string} key
     * @returns Promise
     */
    getMetafield: (productId: number, namespace: string, key: string) => Promise<IMetafield>;
    /**
     * Deletes a metafield by ID for a product.
     *
     * @param  {number} productId
     * @param  {number} metafieldId
     * @returns Promise
     */
    deleteMetafield: (productId: number, metafieldId: number) => Promise<number>;
    /**
     * Deletes a metafields by namespace for a product.
     *
     * @param  {number} productId
     * @param  {number} metafieldId
     * @returns Promise
     */
    deleteMetafieldsByNamespace: (productId: number, namespace: string) => Promise<void>;
}
