"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ErrorHelper_1 = require("./ErrorHelper");
const Logger_1 = require("../Logger");
class ShopifyProductService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.create = (product) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('create', product);
                    const response = yield this.axiosInstance.post('/products.json', { product: product });
                    resolve(response.data.product);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.update = (product, id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`update product with id ${id}`, product);
                    const response = yield this.axiosInstance.put(`/products/${id}.json`, { product: product });
                    resolve(response.data.product);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getAll = () => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('getting all products');
                    const response = yield this.axiosInstance.get('/products/count.json');
                    const numberOfProducts = response.data.count;
                    Logger_1.Logger.info(`Getting ${numberOfProducts} products`);
                    let products = [];
                    while (products.length < numberOfProducts) {
                        let productsGetURL = '/products.json?limit=250&since_id=';
                        if (products.length > 1) {
                            productsGetURL += products[products.length - 1].id;
                        }
                        else {
                            productsGetURL += '0';
                        }
                        Logger_1.Logger.info('Products Get URL  -> ', productsGetURL);
                        const productsRetrieved = yield this.axiosInstance.get(productsGetURL);
                        products = products.concat(productsRetrieved.data.products);
                        Logger_1.Logger.info('Products retrieved -> ' + products.length);
                    }
                    resolve(products);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getCount = () => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('Counting all products');
                    const response = yield this.axiosInstance.get('/products/count.json');
                    const numberOfProducts = response.data.count;
                    Logger_1.Logger.info(`${numberOfProducts} products`);
                    resolve(numberOfProducts);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.get = (num) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getting ${num} products`);
                    let products = [];
                    const response = yield this.axiosInstance.get(`/products/count.json`);
                    const count = response.data.count > num ? num : response.data.count;
                    while (products.length < count) {
                        let productsGetURL = '/products.json?limit=250&since_id=';
                        if (products.length > 1) {
                            productsGetURL += products[products.length - 1].id;
                        }
                        else {
                            productsGetURL += '0';
                        }
                        const productsRetrieved = yield this.axiosInstance.get(productsGetURL);
                        products = products.concat(productsRetrieved.data.products);
                    }
                    resolve(products.slice(0, count));
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getInBatch = (sinceId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Getting products since id ${sinceId}`);
                    const response = yield this.axiosInstance.get('/products/count.json');
                    const count = response.data.count;
                    const batches = parseInt(((count / 250) + 1).toString());
                    const productBatch = { batches: batches, products: [], nextSinceId: 0 };
                    let productsGetURL = `/products.json?limit=250&since_id=${sinceId}`;
                    const productsRetrieved = yield this.axiosInstance.get(productsGetURL);
                    productBatch.products = productsRetrieved.data.products;
                    productBatch.nextSinceId = productBatch.products[productBatch.products.length - 1].id;
                    resolve(productBatch);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * It will only return exact matches.
         */
        this.getByTitle = (title) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getByTitle ${title}`);
                    const response = yield this.axiosInstance.get('/products.json?title=' + encodeURI(title));
                    resolve(response.data.products);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * It will only return exact matches.
         */
        this.getByHandle = (handle) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getByHandle ${handle}`);
                    const response = yield this.axiosInstance.get('/products.json?handle=' + encodeURI(handle));
                    resolve(response.data.products);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getByProductType = (type) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getByProductType ${type}`);
                    const response = yield this.axiosInstance.get('/products.json?product_type=' + type);
                    resolve(response.data.products);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getById = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`getById ${id}`);
                    const response = yield this.axiosInstance.get('/products.json?ids=' + id);
                    resolve(response.data.products ? response.data.products[0] : null);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.delete = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Delete product id ->  ${id}`);
                    yield this.axiosInstance.delete(`/products/${id}.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Gets All metafields of a product
         *
         * @param  {number} productId
         * @returns Promise
         */
        this.getMetafields = (productId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Getting metafields for product id ->  ${productId}`);
                    const response = yield this.axiosInstance.get(`/products/${productId}/metafields.json`);
                    resolve(response.data.metafields);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
        *
        * Adds or updates a metafield.
        *
        * @param  {number} id - product ID
        * @param  {IMetafield} metafield to be saved
        * @returns Promise
        */
        this.saveOrUpdateMetafield = (id, metafield) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Getting metafields for product id ->  ${id}`);
                    const metafieldExisting = yield this.getMetafield(id, metafield.namespace, metafield.key);
                    if (metafieldExisting) {
                        Logger_1.Logger.info(`Metafield exists we will change its value ->  ${metafieldExisting.id}`);
                        metafieldExisting.value = metafield.value;
                        const response = yield this.axiosInstance.put(`/products/${id}/metafields/${metafieldExisting.id}.json`, { metafield: metafieldExisting });
                        resolve(response.data.metafields);
                    }
                    else {
                        const response = yield this.axiosInstance.post(`/products/${id}/metafields.json`, { metafield: metafield });
                        resolve(response.data.metafields);
                    }
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Gets a metafield matching a product namespace and key.
         *
         * @param  {number} productId
         * @param  {string} namespace
         * @param  {string} key
         * @returns Promise
         */
        this.getMetafield = (productId, namespace, key) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Getting metafield for product id ->  ${productId} and namespace ${namespace} and key ${key}`);
                    const metafields = yield this.getMetafields(productId);
                    let foundMetafield = null;
                    if (metafields && metafields.length > 0) {
                        foundMetafield = metafields.find(m => m.namespace === namespace && m.key === key);
                    }
                    resolve(foundMetafield);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Deletes a metafield by ID for a product.
         *
         * @param  {number} productId
         * @param  {number} metafieldId
         * @returns Promise
         */
        this.deleteMetafield = (productId, metafieldId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Deleting metafield id ->  ${metafieldId} for product ${productId}`);
                    yield this.axiosInstance.delete(`/products/${productId}/metafields/${metafieldId}.json`);
                    resolve(metafieldId);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Deletes a metafields by namespace for a product.
         *
         * @param  {number} productId
         * @param  {number} metafieldId
         * @returns Promise
         */
        this.deleteMetafieldsByNamespace = (productId, namespace) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Deleting metafield by namespace ->  ${namespace} for product ${productId}`);
                    const metafields = yield this.getMetafields(productId);
                    const metafieldsToDelete = metafields.filter((m) => m.namespace === namespace);
                    for (const metafieldToDelete of metafieldsToDelete) {
                        yield this.axiosInstance.delete(`/products/${productId}/metafields/${metafieldToDelete.id}.json`);
                    }
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
    }
}
exports.ShopifyProductService = ShopifyProductService;
//# sourceMappingURL=ShopifyProductService.js.map