"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ErrorHelper_1 = require("./ErrorHelper");
const Logger_1 = require("../Logger");
class ShopifyVariantService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.create = (variant, productId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('create', variant);
                    const response = yield this.axiosInstance.post(`/products/${productId}/variants.json`, { variant: variant });
                    resolve(response.data.variant);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.update = (variant, id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`update variant with id ${id}`, variant);
                    const response = yield this.axiosInstance.put(`/variants/${id}.json`, { variant: variant });
                    resolve(response.data.variant);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getCount = (productId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('Counting all variants');
                    const response = yield this.axiosInstance.get(`/products/${productId}/variants/count.json`);
                    const numberOfvariants = response.data.count;
                    Logger_1.Logger.info(`${numberOfvariants} variants`);
                    resolve(numberOfvariants);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getById = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Ge Variant By Id ${id}`);
                    const response = yield this.axiosInstance.get(`/variants/${id}.json`);
                    resolve(response.data.variant ? response.data.variant : null);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.delete = (id, productId) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info(`Delete variant id ->  ${id}`);
                    yield this.axiosInstance.delete(`/products/${productId}/variants/${id}.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
    }
}
exports.ShopifyVariantService = ShopifyVariantService;
//# sourceMappingURL=ShopifyVariantService.js.map