"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Logger_1 = require("../Logger");
const ErrorHelper_1 = require("./ErrorHelper");
const APIVersion_1 = require("./APIVersion");
class ShopifyWebhookService {
    constructor(axiosInstance) {
        this.axiosInstance = axiosInstance;
        this.create = (webhook) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('create webhook', webhook);
                    const response = yield this.axiosInstance.post('/webhooks.json', { webhook: webhook });
                    resolve(response.data.webhooks);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.deleteAll = () => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('delete All webhooks');
                    const webhooks = yield this.getAll();
                    for (const webhook of webhooks) {
                        yield this.axiosInstance.delete(`/webhooks/${webhook.id}.json`);
                    }
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.delete = (id) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('delete webhook with ID ' + id);
                    yield this.axiosInstance.delete(`/webhooks/${id}.json`);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.deleteByTopic = (topic) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const currentWebhooks = yield this.getAll();
                    for (const webhookInLoop of currentWebhooks) {
                        if (webhookInLoop.topic === topic) {
                            yield this.delete(webhookInLoop.id);
                        }
                    }
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        this.getAll = () => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    Logger_1.Logger.info('get All');
                    const response = yield this.axiosInstance.get('/webhooks.json');
                    resolve(response.data.webhooks);
                }
                catch (error) {
                    Logger_1.Logger.error(error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
        /**
         * Rebuilds all webhooks with the current API Version. @link APIVersion
         *
         * @returns Promise
         */
        this.updateWebhooksAPI = () => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const webhooks = yield this.getAll();
                    for (const webhook of webhooks) {
                        if (webhook.api_version !== APIVersion_1.default.VERSION) {
                            const newWebhook = { topic: webhook.topic, format: webhook.format, address: webhook.address };
                            yield this.delete(webhook.id);
                            yield this.create(newWebhook);
                        }
                        else {
                            Logger_1.Logger.info(`Webhooks has the current API version ${APIVersion_1.default.VERSION}`);
                        }
                    }
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error('Error rebuilding webhooks wit the correct API', error, ErrorHelper_1.default.getErrorFromResponse(error));
                    reject(Error(ErrorHelper_1.default.getErrorFromResponse(error)));
                }
            }));
        });
    }
}
exports.ShopifyWebhookService = ShopifyWebhookService;
//# sourceMappingURL=ShopifyWebhookService.js.map