"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const Logger_1 = require("../../Logger");
const __1 = require("../..");
var router = express.Router();
class ShopifyMandatoryRouter {
    constructor(shopifySecret, customerRedactcallback, shopRedactcallback, customersDataCallback) {
        this.shopifySecret = shopifySecret;
        this.customerRedactcallback = customerRedactcallback;
        this.shopRedactcallback = shopRedactcallback;
        this.customersDataCallback = customersDataCallback;
        this.asyncFrame = (req, res, callback) => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                Logger_1.Logger.info('Mandatory webhook', req.body);
                try {
                    const isHmacOk = yield __1.HmacUtils.checkHmac(req, this.shopifySecret);
                    if (!isHmacOk)
                        throw 'Hmac not correct';
                    const response = yield callback(req.body);
                    res.send(response).sendStatus(200);
                    resolve();
                }
                catch (error) {
                    Logger_1.Logger.error(error);
                    res.status(400).send('HMAC validation failed');
                    reject(Error(error));
                }
            }));
        });
    }
    buildRoutes() {
        router.post('/customers/redact', __1.HmacUtils.jsonWebhookParser, (req, res) => { this.asyncFrame(req, res, this.customerRedactcallback); });
        router.post('/customers/data_request', __1.HmacUtils.jsonWebhookParser, (req, res) => { this.asyncFrame(req, res, this.customersDataCallback); });
        router.post('/shop/redact', __1.HmacUtils.jsonWebhookParser, (req, res) => { this.asyncFrame(req, res, this.shopRedactcallback); });
        return router;
    }
}
exports.ShopifyMandatoryRouter = ShopifyMandatoryRouter;
exports.default = ShopifyMandatoryRouter;
//# sourceMappingURL=ShopifyMandatoryRouter.js.map