"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const cookie = require("cookie");
const nonce = require("nonce");
const querystring = require("querystring");
const CryptoService_1 = require("./services/CryptoService");
const OauthService_1 = require("./services/OauthService");
var router = express.Router();
class ShopifyRouterNewScope {
    constructor(shop_secret, shop_key, appBaseUrl, endpointUri, getScopes, callback) {
        this.shop_secret = shop_secret;
        this.shop_key = shop_key;
        this.appBaseUrl = appBaseUrl;
        this.endpointUri = endpointUri;
        this.getScopes = getScopes;
        this.callback = callback;
        this.callBackRoute = (req, res) => {
            const expresRequest = req;
            const expresResponse = res;
            const { shop, hmac, code, state } = expresRequest.query;
            const stateCookie = cookie.parse(expresRequest.headers.cookie).state;
            if (state !== stateCookie) {
                return expresResponse.status(403).send('Request origin cannot be verified');
            }
            if (shop && hmac && code) {
                const map = Object.assign({}, expresRequest.query);
                delete map['signature'];
                delete map['hmac'];
                const message = querystring.stringify(map);
                const hashEquals = this.cryptoService.checkHash(this.cryptoService.generateHash(message, this.shop_secret), hmac);
                if (!hashEquals) {
                    expresResponse.status(400).send('HMAC validation failed');
                    return;
                }
                OauthService_1.OauthService.getOathToken(shop, this.shop_key, this.shop_secret, code, (accessToken) => {
                    this.callback(accessToken, shop, expresResponse, null);
                }, (errorStatus, error) => {
                    this.callback(null, shop, expresResponse, { error: error, errorStatus: errorStatus });
                });
            }
            else {
                expresResponse.status(400).send('Required parameters missing');
            }
        };
        this.installRoute = (req, res) => {
            const expresRequest = req;
            const expresResponse = res;
            const scopes = res.locals.scopes;
            const shop = expresRequest.query.shop;
            if (shop) {
                expresResponse.cookie('state', this.state, { sameSite: 'none', secure: true });
                expresResponse.redirect(this.installUrl(shop, scopes));
            }
            else {
                return expresResponse.status(400).send('Missing shop parameter.');
            }
        };
        this.state = nonce()();
        this.cryptoService = new CryptoService_1.CryptoService();
    }
    buildRoutes() {
        router.get('/', (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            const scopes = yield this.getScopes(req.query.shop);
            res.locals.scopes = scopes;
            next();
        }), this.installRoute);
        router.get('/callback', this.callBackRoute);
        return router;
    }
    installUrl(shop, scopes) {
        return `https://${shop}/admin/oauth/authorize?client_id=${this.shop_key}&scope=${scopes}&state=${this.state}&redirect_uri=${this.redirectUri()}`;
    }
    redirectUri() {
        return `${this.appBaseUrl}${this.endpointUri}/callback`;
    }
}
exports.ShopifyRouterNewScope = ShopifyRouterNewScope;
exports.default = ShopifyRouterNewScope;
//# sourceMappingURL=ShopifyRouterNewScope.js.map