"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const body_parser_1 = require("body-parser");
const CryptoService_1 = require("./CryptoService");
class HmacValidator {
}
HmacValidator.cryptoService = new CryptoService_1.CryptoService();
HmacValidator.jsonWebhookParser = body_parser_1.json({
    type: '*/*',
    limit: '50mb',
    verify: function (req, res, buf) {
        req.rawBody = buf;
    }
});
HmacValidator.getHmac = (req) => {
    return req.headers['x-shopify-hmac-sha256'];
};
HmacValidator.checkHmac = (req, signature) => __awaiter(this, void 0, void 0, function* () {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        try {
            const hmac = HmacValidator.getHmac(req);
            const generated_hmac = HmacValidator.cryptoService.generateHashBase64(req.rawBody, signature);
            const hashEquals = HmacValidator.cryptoService.checkHash(Buffer.from(generated_hmac, 'utf8'), hmac);
            if (!hashEquals) {
                resolve(false);
            }
            else {
                resolve(true);
            }
        }
        catch (err) {
            reject(Error(err));
        }
    }));
});
exports.default = HmacValidator;
//# sourceMappingURL=HmacValidator.js.map