"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const MemoryStores_1 = require("./MemoryStores");
class StoreRateLimiter {
    constructor(windowMs, maxRequests) {
        this.windowMs = 60 * 100;
        this.max = 5;
        this.message = 'Too many requests, please try again later.';
        this.statusCode = 200;
        this.keyGenerator = (req) => {
            return req.headers['x-shopify-shop-domain'];
        };
        this.handler = (req, res) => {
            res.status(this.statusCode).send(this.message);
        };
        this.rateReqLimit = (req, res, next) => {
            const key = this.keyGenerator(req);
            this.store.incr(key, (err, current, resetTime) => {
                if (err) {
                    return next(err);
                }
                req.rateLimit = { limit: this.max, current: current, remaining: Math.max(this.max - current, 0), resetTime: resetTime };
                if (this.max && current > this.max) {
                    return this.handler(req, res);
                }
                else {
                    next();
                }
            });
        };
        this.windowMs = windowMs;
        this.max = maxRequests;
        this.store = new MemoryStores_1.default(this.windowMs);
    }
}
exports.default = StoreRateLimiter;
//# sourceMappingURL=StoreRateLimiter.js.map