"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ShopifyRouter_1 = require("../../shopify/router/ShopifyRouter");
const OauthService_1 = require("../../shopify/router/services/OauthService");
const getScopes = () => __awaiter(this, void 0, void 0, function* () { return 'read_orders'; });
test('install route returns 400 if no shop has been specified', () => {
    const callback = jest.fn();
    const shopifyRouter = new ShopifyRouter_1.default('secret', 'key', 'appBaseUrl', '', getScopes, callback);
    const req = { query: { shop: '' } };
    const res = { status: jest.fn((number) => { return { send: res.send }; }), send: jest.fn(), locals: {} };
    shopifyRouter.installRoute(req, res);
    expect(res.status).toBeCalledWith(400);
});
test('install route returns redirects to oauth url when shop is set', () => {
    const callback = jest.fn();
    const shopifyRouter = new ShopifyRouter_1.default('secret', 'key', 'appBaseUrl', '', getScopes, callback);
    const req = { query: { shop: 'myshop.shopify.com' } };
    const res = { redirect: jest.fn(), cookie: jest.fn(), locals: { scopes: 'read_orders' } };
    shopifyRouter.installRoute(req, res);
    expect(res.cookie).toBeCalledWith('state', expect.anything(), { 'sameSite': 'none', 'secure': true });
    expect(res.redirect).toBeCalledWith(expect.stringContaining('/admin/oauth/authorize?client_id=key&scope=read_orders'));
});
test('callback route returns 403 when origin can not be validated', () => {
    const callback = jest.fn();
    const shopifyRouter = new ShopifyRouter_1.default('secret', 'key', 'appBaseUrl', '', getScopes, callback);
    const req = { query: { state: 'somestate' }, headers: { cookie: 'state=someotherstate' } };
    const res = { status: jest.fn((number) => { return { send: res.send }; }), send: jest.fn(), locals: {} };
    shopifyRouter.callBackRoute(req, res);
    expect(res.status).toBeCalledWith(403);
});
test('callback route returns 400 when shop hmac or code are missing as query params', () => {
    const callback = jest.fn();
    const shopifyRouter = new ShopifyRouter_1.default('secret', 'key', 'appBaseUrl', '', getScopes, callback);
    const req = { query: { state: 'somestate' }, headers: { cookie: 'state=somestate' } };
    const res = { status: jest.fn((number) => { return { send: res.send }; }), send: jest.fn() };
    shopifyRouter.callBackRoute(req, res);
    expect(res.status).toBeCalledWith(400);
});
test('callback route get oauth token when everything is ok', () => {
    const callback = jest.fn();
    const shopifyRouter = new ShopifyRouter_1.default('secret', 'key', 'appBaseUrl', '', getScopes, callback);
    OauthService_1.OauthService.getOathToken = jest.fn();
    shopifyRouter.cryptoService.checkHash = jest.fn(() => true);
    shopifyRouter.cryptoService.generateHash = jest.fn();
    const req = { query: { state: 'somestate', hmac: 'somehmac', code: 'somecode', shop: 'myshop.shopify.com' }, headers: { cookie: 'state=somestate' } };
    const res = { status: jest.fn((number) => { return { send: res.send }; }), send: jest.fn() };
    shopifyRouter.callBackRoute(req, res);
    expect(OauthService_1.OauthService.getOathToken).toBeCalled();
});
test('callback route returns 400 when hmac is not correct', () => {
    const callback = jest.fn();
    const shopifyRouter = new ShopifyRouter_1.default('secret', 'key', 'appBaseUrl', '', getScopes, callback);
    OauthService_1.OauthService.getOathToken = jest.fn();
    shopifyRouter.cryptoService.checkHash = jest.fn(() => false);
    shopifyRouter.cryptoService.generateHash = jest.fn();
    const req = { query: { state: 'somestate', hmac: 'somehmac', code: 'somecode', shop: 'myshop.shopify.com' }, headers: { cookie: 'state=somestate' } };
    const res = { status: jest.fn((number) => { return { send: res.send }; }), send: jest.fn() };
    shopifyRouter.callBackRoute(req, res);
    expect(res.status).toBeCalledWith(400);
});
//# sourceMappingURL=ShopifyRouter.test.js.map