"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const HmacValidator_1 = require("../../../shopify/router/services/HmacValidator");
const CryptoService_1 = require("../../../shopify/router/services/CryptoService");
test('getHmac should get shopify Hmac header', () => {
    const req = { headers: { 'x-shopify-hmac-sha256': 'hmac' } };
    const hmac = HmacValidator_1.default.getHmac(req);
    expect(hmac).toBe('hmac');
});
test('checkHmac resolve to true when hmac is correct', () => {
    const crypto = new CryptoService_1.CryptoService();
    const hmac_generated = crypto.generateHashBase64('someRaw=someRawValue', 'signature');
    const req = { headers: { 'x-shopify-hmac-sha256': hmac_generated }, rawBody: 'someRaw=someRawValue' };
    const hmac = HmacValidator_1.default.checkHmac(req, 'signature');
    expect(hmac).toBeTruthy();
});
test('checkHmac resolve to false when hmac is correct', () => {
    const req = { headers: { 'x-shopify-hmac-sha256': 'wrong_hmac_generated' }, rawBody: 'someRaw=someRawValue' };
    const hmac = HmacValidator_1.default.checkHmac(req, 'signature');
    expect(hmac).toBeTruthy();
});
//# sourceMappingURL=HmacValidator.test.js.map