"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const StoreRateLimiter_1 = require("../../../../shopify/router/services/rateLimiter/StoreRateLimiter");
const next = jest.fn();
afterEach(() => {
    next.mockClear();
});
test('Adds a hit for the store and calls next', (done) => {
    const storeLimiter = new StoreRateLimiter_1.default(1000, 1);
    const req = { headers: { 'x-shopify-shop-domain': 'test.myshopify.com' } };
    const res = {};
    storeLimiter.rateReqLimit(req, res, next);
    setTimeout(() => {
        expect(storeLimiter.store.hits['test.myshopify.com']).toBe(1);
        expect(next).toHaveBeenCalledTimes(1);
        done();
    }, 200);
});
test('Does resturn response when too many requests', (done) => {
    const storeLimiter = new StoreRateLimiter_1.default(1000, 1);
    const req = { headers: { 'x-shopify-shop-domain': 'test.myshopify.com' } };
    const send = jest.fn();
    const res = { status: jest.fn(() => { return { send: send }; }) };
    storeLimiter.rateReqLimit(req, res, next);
    storeLimiter.rateReqLimit(req, res, next);
    setTimeout(() => {
        expect(storeLimiter.store.hits['test.myshopify.com']).toBe(2);
        expect(next).toHaveBeenCalledTimes(1);
        expect(send).toHaveBeenCalledWith('Too many requests, please try again later.');
        done();
    }, 200);
});
test('Does reset all hits after window time', (done) => {
    const storeLimiter = new StoreRateLimiter_1.default(1000, 1);
    const req = { headers: { 'x-shopify-shop-domain': 'test.myshopify.com' } };
    const req1 = { headers: { 'x-shopify-shop-domain': 'test4.myshopify.com' } };
    const res = {};
    storeLimiter.rateReqLimit(req, res, next);
    storeLimiter.rateReqLimit(req1, res, next);
    setTimeout(() => {
        expect(storeLimiter.store.hits['test.myshopify.com']).toBe(1);
        expect(storeLimiter.store.hits['test4.myshopify.com']).toBe(1);
        done();
    }, 200);
});
test('Does adds hits for different stores', (done) => {
    const storeLimiter = new StoreRateLimiter_1.default(1000, 1);
    const req = { headers: { 'x-shopify-shop-domain': 'test.myshopify.com' } };
    const req1 = { headers: { 'x-shopify-shop-domain': 'test4.myshopify.com' } };
    const res = {};
    storeLimiter.rateReqLimit(req, res, next);
    storeLimiter.rateReqLimit(req1, res, next);
    setTimeout(() => {
        expect(storeLimiter.store.hits['test.myshopify.com']).toBeUndefined();
        expect(storeLimiter.store.hits['test4.myshopify.com']).toBeUndefined();
        done();
    }, 1100);
});
//# sourceMappingURL=StoreRateLimiter.test.js.map