interface IRest_Options {
    timeout: number,
    retries: number,
    logger: any
}
interface ITheme {
    id: number;
    name: string;
    created_at: string;
    updated_at: string;
    role: 'main' | 'unpublished';
    previewable: boolean;
}

interface ITemplate {
    key: string;
    public_url: string;
    value: string;
    content_type: 'text/x-liquid';
}

interface IRisk {
    id: number;
    order_id: number;
    checkout_id: number;
    source: string;
    score: string;
    recommendation: string;
    display: boolean;
    cause_cancel: boolean;
    message: string;
    merchant_message: string;
}

interface ILocation {
    id?: number;
    name?: string;
    address1?: string;
    address2?: string;
    city?: string;
    zip?: string;
    province?: string;
    phone?: string;
    legacy_: boolean;
}

interface ICustomer {
    id?: number;
    email: string;
    first_name: string;
    last_name: string;
    phone?: string;
    verified_email: boolean;
    tags?: string;
    note?: string;
    addresses?: IAddress[];
    default_address: IAddress;
    send_email_invite: boolean;
}

interface ICollection {
    id?: number;
    title?: string;
    published?: boolean;
    body_html?: string;
    sort_order?: 'alpha-asc' | 'alpha-desc' | 'best-selling' | 'created' | 'created-desc' | 'manual' | 'price-asc' | 'price-desc';
    image?: { attachment: string };
    metafields?: IMetafield[];
}

interface IMetafield {
    key: string;
    value: string;
    value_type: 'string' | 'integer';
    description: string;
    namespace: string;
    id?: number;
}

interface ICollect {
    id?: number;
    product_id: number;
    collection_id: number;
}

interface IAddress {
    id?: number;
    address1: string;
    address2: string;
    city: string;
    province: string;
    phone?: string;
    zip: string;
    country: string;
    first_name: string;
    last_name: string;
    country_code?: string;
    province_code?: string;
    company?: string;
    default?: boolean;
}

type webhookTopic = 'collections/delete' | 'orders/create' | 'orders/delete' | 'orders/cancelled' | 'orders/fulfilled' | 'orders/paid' | 'orders/partially_fulfilled' | 'orders/updated' | 'customers/create' | 'customers/delete' | 'customers/disable' | 'customers/enable' | 'customers/update' | 'products/create' | 'products/update' | 'products/delete' | 'app/uninstalled' | 'product_listings/add' | 'product_listings/remove' | 'product_listings/update' | 'inventory_items/update' | 'inventory_items/create';

interface IWeebHook {
    topic: webhookTopic;
    address: string;
    format: 'json';
    api_version?: string;
    id?: number;
}

interface IShop {
    email: string;
    shop_owner: string;
}

interface IProduct {
    id?: number;
    title: string;
    body_html: string;
    vendor?: string;
    product_type?: string;
    images?: {src?: string}[];
    tags?: string;
    published_scope?: 'web' | 'global';
    variants?: IVariant[];
    options?: {name: string}[];
    published: boolean;
    created_at?: string;
    published_at?: string;
}

interface IVariant {
    id?: number;
    product_id?: number;
    barcode?: string;
    weight?: number;
    weight_unit?: 'kg';
    inventory_quantity?: number;
    inventory_policy: 'deny' | 'continue';
    price?: string;
    compare_at_price?: string;
    title?: string;
    sku?: string;
    inventory_item_id?: number;
    inventory_management?: 'shopify';
    option1?: string;
    option2?: string;
    option3?: string;
}

interface IInventoryItem {
    id: number;
    sku: string;
    tracked: boolean;
    cost: number;
}

interface IInventoryItemWithVariant extends IInventoryItem {
    variant_price: number;
    variant_id: number;
}

interface IInventoryLevel {
    location_id: number;
    inventory_item_id: number;
    available: number;
}
interface IOrderToCreate {
    note?: string;
    email?: string;
    line_items?: ILineItem[];
    note_attributes?: IAttr[];
    tags?: string;
    customer?: ICustomer;
    financial_status?: 'pending' | 'authorized' | 'partially_paid' | 'paid' | 'partially_refunded' | 'refunded' | 'voided' ;
}

interface IShippingLine {
    code: string;
    price: string;
    price_set?: {
      shop_money: {
        amount: string;
        currency_code: string;
      },
      presentment_money: {
        amount: string;
        currency_code: string;
      }
    },
    discounted_price?: string;
    discounted_price_set?: {
      shop_money: {
        amount: string;
        currency_code: string;
      },
        presentment_money: {
        amount: string;
        currency_code: string;
      }
    },
    source: string;
    title: string;
    tax_lines: any[];
}

interface IOrder {
    id?: number;
    order_number?: number;
    email?: string;
    number?: number;
    note?: string;
    created_at?: string;
    token?: string;
    gateway?: string;
    total_price?: number;
    subtotal_price?: number;
    total_weight?: number;
    total_tax?: number;
    taxes_included?: boolean;
    currency?: 'GBP';
    financial_status?: 'pending' | 'paid' | 'authorized' | 'partially_paid' | 'partially_refunded' | 'refunded' | 'voided';
    confirmed?: boolean;
    total_discounts?: number;
    total_line_items_price?: number;
    cart_token?: string;
    name?: string;
    cancel_reason?: string;
    user_id?: number;
    location_id?: number;
    fulfillment_status?: string;
    line_items?: ILineItem[];
    refunds?: any[];
    fulfillments?: any[];
    shipping_lines?: IShippingLine[];
    shipping_address?: IAddress;
    billing_address?: IAddress;
    order_status_url?: string;
    processing_method?: string;
    note_attributes?: IAttr[];
    tags?: string;
    customer?: ICustomer;
}

type  IOrderStatus  = 'open' | 'closed' | 'cancelled' | 'any';

interface IAttr {
    name: string;
    value: string;
}

interface IProperty {
    name: string;
    value: string;
}

interface ILineItem {
    id: number;
    variant_id?: number;
    title?: string;
    quantity: number;
    price?: number;
    sku?: string;
    fulfillment_status?: string;
    fulfillment_service?: string;
    properties?: IProperty[];
    product_id?: number;
    fulfillable_quantity?: number;
    grams?: number;
    requires_shipping?: boolean;
}

interface IFulFillment {
    id?: number;
    order_id?: number;
    status?: 'pending' | 'open' | 'success' | 'cancelled' | 'error' | 'failure';
    created_at?: string;
    service?: 'manual';
    location_id?: number;
    line_items?: ILineItem[];
    shipment_status?: IShipmentStatus;
    notify_customer?: boolean;
}

interface IFulFillmentLine {
    line_item_id?: number;
    quantity_delivered?: number;
}

interface IFulfillmentService {
    name: string;
    callback_url: string;
    inventory_management: boolean;
    tracking_support: boolean;
    requires_shipping_method: boolean;
    format: 'json';
}

interface IRecurringBillingRequest {
    name: string;
    price: number;
    return_url: string;
    trial_days?: number;
    test: Boolean | null;
}

interface IRecurringBillingResponse {
    id: number;
    name: string;
    api_client_id: number;
    price: number;
    status: string;
    return_url: string;
    billing_on: Date;
    created_at: Date;
    updated_at: Date;
    test: Boolean;
    activated_on: Date;
    trial_ends_on: Date;
    cancelled_on: Date;
    trial_days: number;
    decorated_return_url: string;
    confirmation_url?: string;
}

interface ICustomerRedactRequest {
    shop_id: number;
    shop_domain: string;
    customer: {
        id: number;
        email: string;
        phone: string;
    },
    orders_to_redact: number[]
}

interface IShopRedactRequest {
    shop_id: number;
    shop_domain: string;
}

interface ICustomerRequestData {
    shop_id: number;
    shop_domain: string;
    customer: {
        id: number;
        email: string;
        phone: string;
    },
    orders_requested: number[]
}

interface IPage {
    id?: number;
    title: string;
    shop_id?: number,
    handle?: string;
    body_html: string;
    author?: string;
    created_at?: Date;
    updated_at?: Date;
    published_at?: Date;
    template_suffix: string;
}

interface IProductBatch {
    products: IProduct[];
    batches: number;
    nextSinceId: number;
}

type IShipmentStatus = 'confirmed'| 'in_transit' | 'out_for_delivery' | 'delivered' | 'failure';
type ICustomerRedactCallback = (customerRedactRequest: ICustomerRedactRequest) => Promise<any>;
type IShopRedactCallback = (shopRedactRequest: IShopRedactRequest) => Promise<any>;
type ICustomersDataCallback = (customersDataRequest: ICustomerRequestData) => Promise<IOrder[]>;

interface IShippingProvince {
    id: number;
    country_id: number;
    name: string;
    code: string;
    tax: number;
    tax_name: string;
    tax_type: string;
    tax_percentage: number;
    shipping_zone_id: number;
}

interface IShippingCountry {
    id: number;
    name: string;
    tax: number;
    code: string;
    tax_name: string;
    shipping_zone_id: number;
    provinces: IShippingProvince[];
}

interface IWeightRule {
    id: number;
    name: string;
    price: string;
    shipping_zone_id: number;
    weight_low: number;
    weight_high: number;
}

interface IShippingZone {
    id: number;
    name: string;
    countries: IShippingCountry[];
    weight_based_shipping_rates: IWeightRule[];
    price_based_shipping_rates: any[];
    carrier_shipping_rate_providers: [];
}
